Package: logcondiscr
Type: Package
Title: Estimate a Log-Concave Probability Mass Function from Discrete
        I.i.d. Observations
Version: 1.0.7
Date: 2026-04-26
Authors@R: 
    c(person(given = "Kaspar", family = "Rufibach", role = c("aut", "cre"), email = "kaspar.rufibach@gmail.com"),
    person(given = "Fadoua", family = "Balabdaoui", role = "aut", email = "fadoua.balabdaoui@stat.math.ethz.ch"),
    person(given = "Hanna", family = "Jankowski", role = "aut", email = "hkj@yorku.ca"))
Maintainer: Kaspar Rufibach <kaspar.rufibach@gmail.com>
Depends: Matrix, mvtnorm, cobs
Imports: stats
Description: Given independent and identically distributed observations X(1), ..., X(n), allows to compute the maximum likelihood estimator (MLE) of probability mass function (pmf) under the assumption that it is log-concave, see Weyermann (2007) and Balabdaoui, Jankowski, Rufibach, and Pavlides (2012). The main functions of the package are 'logConDiscrMLE' that allows computation of the log-concave MLE, 'logConDiscrCI' that computes pointwise confidence bands for the MLE, and 'kInflatedLogConDiscr' that computes a mixture of a log-concave PMF and a point mass at k.
License: GPL (>= 2)
URL: http://www.kasparrufibach.ch ,
        https://people.math.ethz.ch/~fadouab/ ,
        https://www.yorku.ca/hkj/
NeedsCompilation: no
Packaged: 2026-04-26 16:30:47 UTC; kaspa
Author: Kaspar Rufibach [aut, cre],
  Fadoua Balabdaoui [aut],
  Hanna Jankowski [aut]
Repository: CRAN
Date/Publication: 2026-04-26 16:40:03 UTC
Built: R 4.5.2; ; 2026-05-01 18:02:33 UTC; unix

## which functions shall be accessible by the users?
export(
    Direction,
    dMLE,
    dTriangular,
    GradientL,
    HesseL,
    J00,
    J10,
    J11,
    J20,
    kInflatedLogConDiscr,
    LikFunk,
    LocalCoarsen,
    LocalConcavity,
    LocalExtend,
    LocalMLE,
    LocalNormalize,
    logConDiscrCI,
    logConDiscrMLE,
    rTriangular,
    StepSize
)
importFrom(stats, qnorm, quantile, rexp, runif)
importFrom(Matrix, bandSparse)
importFrom(mvtnorm, rmvnorm)
importFrom(cobs, conreg)

logConDiscrMLE <- function(x, w = NA, psi_o = NA, prec = 1e-5, output = TRUE){

########################################################################
#
# Compute discrete, concave function psi on [X(1), X(n)] with knots
# only in {X(1), X(2),...,X(m)} such that the log-likelihood function
#     L(psi) <- sum_{i=1}^{m} w(i)*psi(X(i)) 
#              - sum_{j=X(1)}^{X(m)} exp(psi(j))
# is maximal.
#
# INPUT
# - x      : vector of strictly increasing observations
# - w      : vector of weights with entries W_k:= #{i : XRawData_i == X_k}/n
# - psi_o  : concave initialization vector
#
# OUTPUT
# - psi    : vector with entries psi_i := psi(X_i)
# - L      : value of log-likelihood function at maximum
# - isKnot : 0-1-vector where isKnot_k = 1 if psi has a knot at X_k
# - psiSupp: psi interpolated on {X_1, ..., X_m] 
#
# Notwendige m-Files:
# dMLE.m (deshalb auch J00.m, J10.m, J11.m, J20.m)
#
# Matlab version 20.11.07, Kathrin Weyermann
# Ported to R by Kaspar Rufibach, October 2010
########################################################################

# prepare data
if (identical(w, NA)){
    X <- unique(sort(x))
    W <- as.vector(table(x) / length(x))
    }
    
if (identical(w, NA) == FALSE){
    w <- as.numeric(w)
    W <- w[order(x)]
    X <- sort(x)
    W <- w / sum(w)
    }    

# initalize
iterActive1 <- 0
dX <- diff(X)
m <- length(X)
isKnot <- rep(0, m)
if (identical(psi_o, NA)){psi_o <- LocalNormalize(rep(1, m), dX)} else {
    psi_o <- LocalNormalize(psi_o, dX)  
    conc <- LocalConcavity(psi_o, dX)
    isKnot <- as.numeric(abs(conc) > prec)
    }
isKnot[1] <- 1 
isKnot[length(isKnot)] <- 1

# active set algorithm
res <- LocalMLE(X, W, isKnot, psi_o, prec)
psi <- res$psi
L <- res$L
conc <- res$conc
D <- res$D

# basic procedure 2
while (((max(D) > prec * mean(abs(D)))) & (iterActive1 < 500)){
    iterActive1 <- iterActive1 + 1
    isKnot_old <- isKnot
    #Knoten hinzufuegen:
    k <- which.max(D)
    isKnot[k] <- 1    
    res <- LocalMLE(X, W, isKnot, psi, prec)
    psi_new <- res$psi
    L <- res$L
    conc_new <- res$conc
    D <- res$D    
    
    # basic procedure 1
    iterActive2 <- 0
    while (max(conc_new) > prec * max(abs(conc_new))){

        # remove non-concave knots
        JJ <- which(conc_new > 0)  
        conc_diff <- conc[JJ] - conc_new[JJ]
        
        # avoid division by 0
        conc_diff[conc_diff == 0] <- prec ^ 2    
        tmp <- conc[JJ] / conc_diff
        lambda <- min(tmp)
        KK <- which(tmp == lambda)
        isKnot[JJ[KK]] <- 0
        psi <- (1 - lambda) * psi + lambda * psi_new
        conc <- pmin(LocalConcavity(psi, dX), 0)
        res <- LocalMLE(X, W, isKnot, psi, prec)
        psi_new <- res$psi
        L <- res$L
        conc_new <- res$conc
        D <- res$D
        iterActive2 <- iterActive2 + 1
        
        if (identical(output, TRUE)){cat(paste("Outer active set iteration:", format(iterActive1, width = 3), " / inner iteration:", format(iterActive2, width = 2), "\n", sep = ""))}
    } # end while
    psi <- psi_new
    conc <- conc_new
    
    # avoid endless loop if set of knots does not change
    if (identical(isKnot, isKnot_old)){D <- rep(0, length(D))}

    if (identical(output, TRUE)){cat(paste("Outer active set iteration:", format(iterActive1, width = log10(iterActive1) + 2), "\n", sep = ""))}
    } # end while
    
# interpolate estimate on {X_1, ..., X_m}
II <- NULL
XSupp <- X[1]:X[length(X)]   
for (i in 1:length(X)){II <- c(II, which(XSupp == X[i]))}
isObs <- rep(0, length(XSupp))
isObs[II] <- 1
psiSupp <- LocalExtend(XSupp, isObs, X, psi)

# collect and output results
res <- list("x" = X, "w" = W, "psi" = psi, "L" = L, "isKnot" = isKnot, "xSupp" = XSupp, "psiSupp" = psiSupp)
return(res)
}

logConDiscrCI <- function(dat, conf.level = 0.95, type = c("MLE", "all")[1], B = 1e3, output = TRUE, seed = 2011){

    dat <- sort(dat)
    alpha		<-	1 - conf.level

    ## log-concave PMF
    mle 		<-	logConDiscrMLE(dat, output = FALSE)  # find MLE
    p.hat 		<-	exp(mle$psiSupp)                     # MLE of pmf
    supp 		<-	min(dat):max(dat)                    # support of MLE
    knotsMLE 	<-	mle$x[mle$isKnot == 1]               # locations of knots   
    n 			<-	length(dat)

    ## empirical PMF
    emp 		<-	as.numeric(table(dat) / length(dat))
    x.emp		<-	as.numeric(names(table(dat)))
    
    set.seed(seed)
    
    ## determine set of knots according to chosen option
    mm			<-	length(supp)
    if (identical(type, "all")){knots <- supp}
    if (identical(type, "MLE")){knots <- knotsMLE}
    #knots		<-	unique(sort(c(knots, mm + min(dat))))
    lp			<-	rep(0, mm)
    up			<-	rep(0, mm)

    ## run simulations on each interval
    for (j in 1:(length(knots) - 1)){
	   r 		<-	knots[j]
	   s		<-	knots[j + 1]
       if(j == (length(knots) - 1)){s <- s + 1}
	   ind		<-	r:(s-1)
	   p_rs		<-	p.hat[ind - min(dat) + 1]
	   m		<-	s - r
	   if(m > 1){ 
	   	
	  		sigma 	<-	diag(p_rs) - t(t(p_rs)) %*% t(p_rs)
			GGp		<-	matrix(0, B, m)

			for(i in 1:B){
            	cx 		<-	mvtnorm::rmvnorm(1, mean = rep(0, m), sigma = sigma, method = c("eigen", "svd", "chol")[3])
            	Res 	<-	cobs::conreg(x = ind, y = cx/p_rs, w = p_rs)
            	GGp[i,]	<-	p_rs * Res$yf
            	if(identical(output, TRUE) & (round(i / 100) == i / 100)){print(paste("knot segment ", j, " of ", length(knots) - 1, " / run ", i, " of ", B, sep = ""))}
        		} # end for loop

        	## compute quantiles
        	lp1		<- apply(GGp, 2, quantile, (1 - conf.level) / 2)
        	up1		<- apply(GGp, 2, quantile, (1 + conf.level) / 2)

       		} # end if for (m > 1)
        

	   if(m == 1){ 
	   		sigma	<-	p_rs - (p_rs) ^ 2 
	   		lp1		<-	-qnorm(p = 1 - alpha / 2) * sqrt(sigma)
	   		up1		<-	qnorm(p = 1 - alpha / 2) * sqrt(sigma)
	   		}
        
        ## collect quantiles
       	lp[ind - min(dat) + 1] <- lp1
        up[ind - min(dat) + 1] <- up1
    }
    
    lci <- p.hat - up / sqrt(n)  	# HERE
    uci <- p.hat - lp / sqrt(n)	# HERE
    lp  <- pmax(lci, 0)			# HERE
    up  <- pmin(uci, 1)			# HERE
            
    ## generate output
    res <- list("MLE" = mle, "emp" = cbind("x.emp" = x.emp, "emp" = emp), "CIs" = cbind(supp, lp, up))
    return(res)
}

LocalCoarsen <- function(X, W, isKnot){

# Re-distributes X and W on knot points
#
# OUTPUT
# X2 : X(isKnot == 1)
# W2 : W re-weighted on W(isKnot == 1) 

KK <- which(isKnot == 1)
X2 <- X[KK]
W2 <- W[KK]
for (k in 1:(length(KK) - 1)){

    if (KK[k + 1] > (KK[k] + 1)){

        # indices of observations between two knots
        ind <- (KK[k] + 1) : (KK[k + 1] - 1)
        
        # relative distance of these observations to left knot
        lambda <- (X[ind] - X2[k]) / (X2[k + 1] - X2[k])

        # weights of "no knots" re-distributed to closest "is knot"
        W2[k]     <- W2[k]     + sum(W[ind] * (1 - lambda))
        W2[k + 1] <- W2[k + 1] + sum(W[ind] * lambda)
    } # end if
} # end for

res <- list("X2" = X2, "W2" = W2)
return(res)
}

LocalConcavity <- function(psi, dX){

# OUTPUT
# conc : vector with entries conc_k, where conc_k is the difference of slopes
#        to the left and right of psi_k, i.e.
#        conc[k] <= 0 --> psi[k] is a "point of concavity",
#        conc[k] => 0 --> psi[k] is a "point of convexity"
#        where conc_1 = conc_m := 0

m <- length(psi)
conc <- rep(0, m)

# deriv_k := slope between psi_k und psi_(k + 1)
if (m > 2){
    deriv <- (psi[2:m] - psi[1:(m - 1)]) / dX
    conc[2:(m - 1)] <- diff(deriv)
    }
    
return(conc)
}

J00 <- function(fx, fy, d){

########################################################################
# Compute auxiliary function J as described in Weyermann (2008):
#
# J00(fx, fy, d) := sum_{j = 0} ^ {d} exp[(1 - j / d) * fx + (j / d) * fy]
# 
# where d = y - x for integers x < y.
# 
# Matlab version 20.11.07, Kathrin Weyermann
# Ported to R by Kaspar Rufibach, October 2010
########################################################################

# initialization
z <- rep(1, length(fx))
df <- fx - fy
Q <- exp(-abs(df) / d)

# fx = fy    
noStep <- which(df == 0)
if (length(noStep) > 0){z[noStep] <- exp(fx[noStep]) * (d[noStep] + 1)}

# fx != fy
isStep <- which(df != 0) 

if (length(isStep) > 0){
    
    # use Taylor approximation if |1 - Q| < 10^-10
    TT <- which(abs(1 - Q[isStep]) < 1e-10)
    if (length(TT) > 0){
        dT <- d[isStep[TT]]
        R <- Q[isStep[TT]] - 1
        z[isStep[TT]] <- exp(pmax(fx[isStep[TT]], fy[isStep[TT]])) * (dT + 1 + R * (dT ^ 2 + dT) / 2 + R ^ 2 * (dT ^ 3 - dT) / 6)
        }

    # exact if |Q - 1| is not small
    EE <- which(abs(1 - Q[isStep]) >= 1e-10)    
    if (length(EE) > 0){
        dE <- d[isStep[EE]]
        Q <- Q[isStep[EE]]
        z[isStep[EE]] <- exp(pmax(fx[isStep[EE]], fy[isStep[EE]]) ) * (1 - Q ^ (dE + 1)) / (1 - Q)
        }
    }
    
return(z)
}

dMLE <- function(X, W, psi_o = NA, prec = 1e-7){

# dMLE computes vector psi s.t. the function L, defined as
#
# L(psi) := sum(W * psi) - sum_{k = 1} ^ {m - 1} J(psi_k, psi_(k + 1)) 
#                     + sum_{k = 2} ^ {m - 1} exp(psi_k)
#
# is maximized.
# 
# INPUT
# - X     : vector (X_1 < ... < X_m) of observations
# - W     : vector of weights that sum to one
# - psi_o : initialization vector (optional)
# 
# OUTPUT
# - psi   : vector that maximizes L below
# - L     : value of L at psi
#
#
# Version 17.10.07, Kathrin Weyermann
# Ported to R by Kaspar Rufibach, October 2010
########################################################################

# initialization
m <- length(X)
dX <- diff(X)
if (identical(psi_o, NA)){psi_o <- log(1 / m[1]) * rep(1, m)}
psi <- psi_o 
L <- LikFunk(W, psi, dX) 
iterDMLE <- 0
stepDirection <- Direction(W, psi, dX)
norm2 <- sum(stepDirection ^ 2)

while ((norm2 > prec) & (iterDMLE < 50)){
    iterDMLE <- iterDMLE + 1
    stepSize <- StepSize(W, psi, dX, stepDirection)
    psi <- psi + stepSize * stepDirection
    L <- LikFunk(W, psi, dX) 
    stepDirection <- Direction(W, psi, dX)
    norm2 <- sum(stepDirection ^ 2)
} # end while

res <- list("psi" = psi, "L" = L)
return(res)
}

HesseL <- function(psi, dX){

# Compute Hesse matrix of log-likelihood L at psi

m <- length(psi)

# main diagonal
diag <- rep(0, m)
diag[1] <- -J20(psi[1], psi[2], dX[1])
diag[m] <- -J20(psi[m], psi[m - 1], dX[length(dX)]) 
if (m > 2){
    tmp1 <- J20(psi[2:(m - 1)], psi[1:(m - 2)], dX[1:(length(dX) - 1)])
    tmp2 <- J20(psi[2:(m - 1)], psi[3:m], dX[2:length(dX)])
    diag[2:(m - 1)] <- - tmp1 - tmp2 - exp(psi[2:(m - 1)])
    }

# off diagonals
tmpDiag <- -J11(psi[1:(m - 1)], psi[2:m], dX[1:length(dX)])
obDiag <- c(0, tmpDiag)
unDiag <- c(tmpDiag, 0)

# collect results in matrix
hesseL <- diag(m) * 0
hesseL <- Matrix::bandSparse(n = m, k = -1:1, diagonals = list(unDiag, diag, obDiag[-1]))
return(hesseL)
}

J11 <- function(fx, fy, d){

# Computes first partial derivative of J w.r.t. to fx and fy

z <- rep(0, length(fx))
df <- fx - fy
Q <- exp(-abs(df) / d)

# fx = fy    
noStep <- which(df == 0)
z[noStep] <- exp(fx[noStep]) * (d[noStep] - 1) * (d[noStep] + 1) / (6 * d[noStep])

# fx != fy
isStep <- which(df != 0) 

# Taylor approximation if |1-Q| < 10^-3
TT <- which(abs(1 - Q[isStep]) < 1e-3)
if (length(TT) > 0){
    dT <- d[isStep[TT]]
    R <- Q[isStep[TT]] - 1
    z[isStep[TT]] <- exp(max(fx[isStep[TT]], fy[isStep[TT]])) * 
        ((dT ^ 2 - 1) / (6 * dT) 
        + R * (dT ^ 2 - 1) / 12 
        + R ^ 2 * (dT ^ 2 - 1) * (dT - 2) * (3 * dT + 1) / (120 * dT)
        + R ^ 3 * (dT ^ 2 - 1) * (dT - 2) * (dT - 3) * (2 * dT + 1) / (360 * dT))
    }

# exact if |Q - 1| is not small
EE <- which(abs(1 - Q[isStep]) >= 1e-3)    
if (length(EE) > 0){
    dE <- d[isStep[EE]]
    Q <- Q[isStep[EE]]
    z[isStep[EE]] <- exp(pmax(fx[isStep[EE]], fy[isStep[EE]])) * ((dE - 1) * Q ^ (dE + 2) - (dE + 1) * Q ^ (dE + 1) + (dE + 1) * Q ^ 2 + (1 - dE) * Q) / (dE ^ 2 * (Q - 1) ^ 3) 
    }

return(z)
}

Direction <- function(W, psi, dX){

# Output:
# stepDirection : vector that points in direction of max L(psi) via Newton step

mat <- as.matrix(HesseL(psi, dX))
stepDirection <- - solve(mat) %*% GradientL(W, psi, dX)
return(stepDirection)
}

GradientL <- function(W, psi, dX){

# Computes gradient of the log-likelihood L at psi
m <- length(psi)
tmp <- rep(NA, m)
tmp[1] <- J10(psi[1], psi[2], dX[1])
tmp[m] <- J10(psi[length(psi)], psi[length(psi) - 1], dX[length(dX)])

if (m > 2){
    J <- J10(psi[2:(m - 1)], psi[3:m], dX[2:length(dX)]) + J10(psi[2:(m - 1)], psi[1:(m - 2)], dX[1:(length(dX) - 1)]) - exp(psi[2:(m - 1)])
    tmp[2:(m - 1)] <- J
    }

gradL <- W - tmp
return(gradL)
}

dTriangular <- function(a, b, c, d, e){

    ## computes the density at all x in {a, ..., d}
    x <- seq(a, d, by = 1)
    n <- length(x)
    f <- rep(NA, n)

    # left part
    ind1 <- (1:n)[(x >= a) & (x < b)]
    f[ind1] <- c / (b - a) * (x[ind1]  - a)

    # right part
    ind2 <- (1:n)[(x >= b) & (x <= d)]
    f[ind2] <- (e - c) / (d - b) * (x[ind2] - b) + c

    ## f is the log-density
    dens <- exp(f)
    dens <- dens / sum(dens)
    return(dens)
}

LocalExtend <- function(X, isKnot, X2, psi2){

# extrapolates psi2 linearly between knots

KK <- which(isKnot == 1)
psi <- rep(0, length(X))
psi[KK] <- psi2

for (k in 1:(length(KK) - 1)){
    if (KK[k + 1] > (KK[k] + 1)){
        ind <- (KK[k] + 1):(KK[k + 1] - 1)
        lambda <- (X[ind] - X2[k]) / (X2[k + 1] - X2[k])

        # psi("not knot") = linearly interpolated from neighboring knots
        psi[ind] <- (1 - lambda) * psi2[k] + lambda * psi2[k + 1]
    }
}

return(psi)
}

## function to generate random numbers
rTriangular <- function(n, a, b, c, d, e){

    ## generate random numbers according to Devroye's algorithm
    ## from the triangular density

    x <- seq(a, d, by = 1)
    dens <- dTriangular(a, b, c, d, e)
    logdens <- log(dens)

    rand <- rep(NA, n)
    m <- b     ## location of the mode
    pm <- dens[x == m]
    w <- 1 + pm / 2

    for (i in 1:n){

        cond <- TRUE

        while (identical(cond, TRUE)){
            U <- runif(1)
            W <- runif(1)
            S <- 2 * round(runif(1)) - 1
            if (U <= (w / (1 + w))){Y <- runif(1) * w / pm} else {Y <- (w + rexp(1)) / pm}
            X <- S * round(Y)

            ## now choose the correct value of the density once m + X is in {a, ..., d}
            densXm <- 0
            if (((m + X) >= a) & ((m + X) <= d)){densXm <- dens[x == (m + X)]}
            cond <- (W * min(1, exp(w - pm * Y))) > (densXm / pm)
        }

        rand[i] <- m + X
        #if (i / 1000 == round(i / 1000)){print(i)}
    }

res <- list("rand" = rand, "x" = x, "dens" = dens)
return(res)
}

kInflatedLogConDiscr <- function(x, k = 0, prec1 = 1e-10, prec2 = 1e-15, itermax = 200, output = TRUE, theta0 = 0.5, p0 = NA){

	n <- length(x)
	out	<- sort(unique(x))
	z <- min(out):max(out)
    m <- length(z)	

	if((k %in% z) == FALSE){stop("k must be in the set {x_1, x1 + 1, ..., x_n - 1, x_n}")}
    if (identical(p0, NA)){p0 <- rep(1, diff(range(x)) + 1) / (diff(range(x)) + 1)}
    
    ## empirical on x1, x1 + 1, ..., xn - 1, xn
	emp <- rep(0, length(z))
	emp[out - min(x) + 1] <- as.vector(table(x) / n)

	theta.old <- theta0
	p.old <- p0
	w <- emp
    Eloglik.old <- prec1 + 1

	for(i in 1:itermax){

		theta.new <- emp[k - min(x) + 1] * theta.old / (theta.old + (1 - theta.old) * p.old[k - min(x) + 1]) 
		w[k - min(x) + 1] <- emp[k - min(x) + 1] - theta.new  

        # protect logConDiscrMLE() against instabilities due to zero values in w at the ends
		loc <- min(which(w > prec2)):max(which(w > prec2)) 
 
		mle <- logConDiscrMLE(z[loc], w = w[loc], output = FALSE)
		p.new <- rep(0, length(z))
		#p.new[mle$x - min(x) + 1] <- exp(mle$psiSupp)
        p.new[min(mle$x):max(mle$x) - min(x) + 1] <- exp(mle$psiSupp)
		Eloglik.new <- log(theta.new) * theta.new + log(1 - theta.new) * (1 - theta.new) + sum(w[p.new > 0] * log(p.new[p.new > 0]))
        Eloglik.diff <- Eloglik.new - Eloglik.old
        
      	if (identical(output, TRUE)){print(paste("iteration: ", format(i, digits = ceiling(log(itermax) / log(10))), " / E[comploglik]: ", format(Eloglik.new, digits = 4), " / diff loglik: ", format(Eloglik.diff, digits = 4), " / theta: ", format(theta.new, digits = 4), sep = ""))}
		if(abs(Eloglik.diff) < prec1){break}

		theta.old <- theta.new
		p.old <- p.new 
		Eloglik.old <- Eloglik.new
}

	q <- rep(0, m)
	q[k - min(x) + 1] <- 1
	f.fit <- theta.new * q + (1 - theta.new) * p.new
	res <- list("z" = z, "f" = f.fit, "E(L)" = Eloglik.new, "loglik" = sum(emp[f.fit > 0] * log(f.fit[f.fit > 0])), "theta" = theta.new, "logconc.pmf" = p.new, "logconc.z" = mle$x)
	return(res)
}

LocalNormalize <- function(psi, dX){

# normalize psi s.t. the Lagrange term of L is 1

m <- length(psi)
last <- 0
if (m > 2){last <- sum(exp(psi[2:(m - 1)]))}
lagrangeTerm <- sum(J00(psi[1:(m - 1)], psi[2:m], dX)) - last
psi_new <- psi - log(lagrangeTerm)

return(psi_new)
}

LikFunk <- function(W, psi, dX){

# Computes value of log-likelihood at psi where
#
# L(psi) := sum(W * psi) - sum_{X_1} ^ {X_n} exp(psi(z))
#         = sum(W * psi) - sum_{k = 1} ^ {m - 1} J(psi_k, psi_(k+1))
#           + sum_{k = 2} ^ {m - 1} exp(psi_k)

m <- length(psi)
last <- 0
if (m > 2){last <- sum(exp(psi[2:(m - 1)]))}
L <- sum(W * psi) - sum(J00(psi[1:(m - 1)], psi[2:m], dX)) + last

return(L)
}

StepSize <- function(W, psi, dX, stepDirection){

# binary search: to = 2^-M for smallest positive M s.t. H(to) >= 0
#
# OUTPUT
# t : optimal step length in (0,1] in direction "stepDirection"
#     via Hermite interpolation for convex functions
# 
# function 'HelpFunk' from Matlab implementation directly included here

# search for optimal to
to <- 1
Hto <- LikFunk(W, psi + to * stepDirection, dX) - LikFunk(W, psi, dX)
while (Hto < 0){
    to <- to / 2
    Hto <- LikFunk(W, psi + to * stepDirection, dX) - LikFunk(W, psi, dX)
    }

# step length
dH0 <- t(GradientL(W, psi, dX)) %*% stepDirection
dH0to <- dH0 * to
if (Hto >= dH0to / 2){t <- to} else {t <- to * dH0to / (2 * (dH0to + Hto))}

return(as.numeric(t))
}

LocalMLE <- function(X, W, isKnot, psi_o, prec){

########################################################################
#
# Compute concave/convex knots and auxiliary vector D
# where D(a) := DL(psi, Delta_a), so the directional derivative in direction
# "new knot at X_a".
#
# Delta_a is defined as
#
# Delta_a(X_j) = 0                                      if X_j is outside X_KK[i], X_KK[i + 1] 
# Delta_a(X_j) ~ (X_j - X_KK[i]) / (X_k - X_KK[i])      if X_j in [X_KK[i], X_k]
# Delta_a(X_j) ~ (X_KK[i] - X_j) / (X_KK[i + 1] - X_k)  if X_j in [X_k, X_KK[i + 1]]
#     
# So
#   D(a) = sum_{j = KK[i]} ^ {KK[i + 1]} wtmp(j) * delta_a(X_j)
# where
#   wtmp = W(ind(2:end - 1))
#          - J10( psi(ind(2:end - 1)), psi(ind(1:end - 2)), dX(ind(1:end - 2)))
#          - J10( psi(ind(2:end - 1)), psi(ind(3:end)), dX(ind(2:end - 1)))
#          + exp( psi(ind(2:end - 1)))
#
# Matlab version 20.11.07, Kathrin Weyermann
# Ported to R by Kaspar Rufibach, October 2010
########################################################################

# initialization
res1 <- LocalCoarsen(X, W, isKnot)
X2 <- res1$X2
W2 <- res1$W2
res2 <- dMLE(X2, W2, psi_o[which(isKnot == 1)])
psi2 <- res2$psi
L <- res2$L
psi <- LocalExtend(X, isKnot, X2, psi2)

# determine concave/convex knots
dX <- diff(X)
conc <- isKnot * LocalConcavity(psi, dX)

# auxiliary vector D
D <- rep(0, length(X))
KK <- which(isKnot == 1)

for (i in 1:(length(KK) - 1)){
    if (KK[i + 1] > (KK[i] + 1)){
        dtmp <- X[KK[i + 1]] - X[KK[i]]
        
        # indices between two knots, incl. the knots
        ind <- KK[i]:KK[i + 1]
        xtmp <- (X[ind[2:(length(ind) - 1)]] - X[KK[i]]) / dtmp
        tmp_grad <- GradientL(W[ind], psi[ind], dX[ind[1:(length(ind) - 1)]])
        wtmp <- tmp_grad[2:(length(tmp_grad) - 1)]
        D[ind[2:(length(ind) - 1)]] <- dtmp * (cumsum(wtmp * xtmp) - xtmp * cumsum(wtmp) + xtmp * sum(wtmp * (1 - xtmp)))
    }
}

# generate output
res <- list("psi" = psi, "L" = L, "conc" = conc, "D" = D)
return(res)

}

\name{logConDiscrMLE}
\alias{logConDiscrMLE}
\title{Compute log-concave probability mass function from i.i.d. data}
\description{Compute the maximum likelihood estimate of a log-concave probability mass function from discrete i.i.d. data.}
\usage{logConDiscrMLE(x, w = NA, psi_o = NA, prec = 1e-05, output = TRUE)}
\arguments{
  \item{x}{Vector of observations. If \code{w = NA}, then weights will be generated for each non-unique observation of \code{x}.}
  \item{w}{If \code{w = NA}, weights will be generated from \code{x}. If \code{w != NA}, then it is assumed that \code{x} and \code{w} are of equal length and 
  the elements in \code{w} correspond to the weights in \code{x}.}
  \item{psi_o}{Optional start vector.}
  \item{prec}{Precision for stopping criterion.}
  \item{output}{Logical, if \code{TRUE}, progress of the active set algorithm is shown.}
}
\details{Given a vector of observations \eqn{{\bold{x}_n} = (x_1, \ldots, x_n)}{x_n = (x_1, \ldots, x_n)} from a discrete PMF,
\code{\link{logConDiscrMLE}} computes a function \eqn{\widehat p_k}{\widehat p_k} on \eqn{\{x_1, \ldots, x_n\}} with knots only 
in \eqn{\{x_1, \ldots, x_n\}}{\{x_1, \ldots, x_n\}} such that 
  
  \deqn{L(\bold{p}) = \sum_{i=1}^n w_i \log(p_i)}
  
  is maximal over all log-concave PMFs \eqn{\{p_k\}, k=1, \ldots, n}, where \eqn{w_i} is the frequency of the observation \eqn{x_i}. 
  To accomplish this, an active set algorithm is used.}
\value{A list containing the following elementes:
\item{x}{Vector of unique observations, sorted.}
\item{w}{Generated weights.}
\item{psi}{The estimated log-density on the grid of unique, sorted observations.}
\item{L}{The value of the log-likelihood at the maximum.}
\item{IsKnot}{Binary vector where \code{isKnot_k = 1} if \eqn{\psi} has a knot at \eqn{x_k}.}
\item{xSupp}{The full support \eqn{\{x_1, x_1 + 1, \ldots, x_m - 1, x_m\}}.}
\item{psiSupp}{\eqn{\psi} interpolated on \code{xSupp}.}
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Weyermann, K. (2007).
An Active Set Algorithm for Log-Concave Discrete Distributions.
\emph{MSc thesis, University of Bern} (Supervisor: Lutz Duembgen).
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\examples{

# -------------------------------------------------------------
# compute MLE for a random sample from a Poisson distribution
# -------------------------------------------------------------
x <- sort(rpois(n = 100, lambda = 2))
mle <- logConDiscrMLE(x)
psi <- mle$psi

# plot estimated PMF and log of estimate
par(mfrow = c(1, 2), las = 1)
true <- dpois(0:20, lambda = 2)
plot(mle$x, exp(psi), type = "p", col = 2, xlim = range(x), xlab = "x", 
    ylim = c(0, max(exp(psi), true)), ylab = "PMF", 
    main = "Estimate MLE from a Poisson", pch = 19)
legend("topright", c("truth", "MLE"), col = c(4, 2), lty = c(1, 0), 
    pch = c(0, 19), bty = "n")

# add true PMF
lines(0:20, true, type = "l", col = 4)

# log-density
plot(mle$x, psi, type = "p", col = 2, xlim = range(x), xlab = "x", 
    ylab = "PMF", main = "Estimate MLE from a Poisson", pch = 19)
lines(0:20, log(true), type = "l", col = 4)

# use a priori specified weights: mle = mle2
mle2 <- logConDiscrMLE(x = unique(x), w = table(x))


## -------------------------------------------------------------
## Illustrate the limit process: the code below can be used to
## to reproduce the limit process figure in Balabdaoui et al (2011)
## -------------------------------------------------------------
a <- 1
b <- 7
c <- 8
d <- 11
e <- 2
n <- 10 ^ 2

## support
x <- seq(a, d, by = 1)

## true density
dens <- dTriangular(a, b, c, d, e)
logdens <- log(dens)
rand <- rTriangular(n, a, b, c, d, e)$rand

## empirical
emp <- table(rand) / n
x.emp <- names(table(rand))

## log-concave MLE
mle <- logConDiscrMLE(rand, output = FALSE)

## plot log PMF and PMF
par(mfrow = c(1, 2))
plot(x, logdens, type = "l", col = 1, pch = 19, main = "log-density", 
    xlim = c(a, d), ylim = range(range(log(emp), logdens)))
lines(x, logdens, type = "l", col = 1, lwd = 0.1)
points(x.emp, log(emp), col = 4, pch = 19)
points(mle$x, mle$psi, col = 6, pch = 19)
abline(v = mle$x[mle$isKnot == 1], lty = 3, col = 3)

plot(x, dens, type = "l", col = 1, pch = 19, main = "density", 
    xlim = c(a, d), ylim = c(0, max(dens, emp)))
lines(x, dens, type = "l", col = 1, lwd = 0.1)
points(x.emp, emp, col = 4, pch = 19)
points(mle$x, exp(mle$psi), col = 6, pch = 19)
legend("topleft", c("truth", "MLE", "knots of the MLE", "empirical"), 
    col = c(1, 6, 3, 4), pch = c(NA, 19, NA, 19), lty = c(1, NA, 3, NA), 
    bg = "white", bty = "n")
abline(v = mle$x[mle$isKnot == 1], lty = 3, col = 3)


## -------------------------------------------------------------
## Now compute and plot Y(x) and H(x)
## -------------------------------------------------------------
xla <- paste("x = {r = ", a, ", ..., s - 1 = ", b - 1, "}", sep = "")
par(mfcol = c(2, 2), oma = rep(0, 4), mar = c(4.5, 4.5, 2, 1), las = 1)
plot(x, logdens, type = "b", col = 2, pch = 19, main = "log of 
    normalized triangular pmf", xlim = c(a, d), xaxt = "n", xlab = "x", 
    ylab = "log of normalized pmf")
axis(1, at = c(a, b, d), labels = paste(c("a = ", "b = ", "d = "), 
    c(a, b, d), sep = ""))

## compute H(x)
r <- a
s <- b
ind <- r:(s - 1)
px <- dens
p_rs <- px[ind]
m <- s - r

## -------------------------------------------------------------
## generate one observation from the distribution of U(F(x)) - U(F(x - 1))
## -------------------------------------------------------------
sigma <- diag(m) * 0
for (i in 1:m){
    for (j in 1:m){
        sigma[i, j] <- p_rs[i] * (i == j) - p_rs[i] * p_rs[j]
    }
}

set.seed(23041977)
cx <- rep(NA, d - a + 1)
cx[ind] <- rmvnorm(1, mean = rep(0, m), sigma = sigma, method = 
    c("eigen", "svd", "chol")[3])
Ux <- rep(NA, length(x))
Ux[ind] <- cx[ind]

X <- x[ind]
Y <- Ux[ind] / p_rs
W <- p_rs

## concave regression using 'cobs'
Res <- conreg(x = X, y = Y, w = W, verbose = TRUE)
g <- Res$yf
gKnots <- Res$iKnots
plot(X, Y, main = expression("The concave function g* that 
    minimizes "*Phi*"(g)"), xaxt = "n", ylab = "g*", ylim = 
    range(c(Y, g)), xlab = xla, type = "n")
axis(1, at = 0:100, labels = 0:100); abline(v = x[gKnots], 
    lty = 2, col = grey(0.75))
lines(X, g, lwd = 2, col = 3, type = "b", pch = 1)
lines(X, Y, lwd = 1, col = 2, type = "p", pch = 19)
legend("bottomright", c("values of cx / px", "minimizer g*"), 
    lty = c(NA, 1), pch = c(19, 1), col = 2:3, bty = "n", 
    lwd = c(NA, 2))

## compute H(x) for x = r, ..., s - 1 and plot it
gstar <- rep(NA, length(x))
gstar[ind] <- g
xs <- r:(s - 1)
Hs <- rep(0, length(xs))
for (i in 2:length(xs)){
    for (ks in r:(xs[i] - 1)){
        js <- r:ks
        Hs[i] <- Hs[i] + sum(gstar[js] * px[js])
    }
}

## check
(Hs[3:length(Hs)] - 2 * Hs[2:(length(Hs) - 1)] + Hs[1:(length(Hs) - 2)]) / p_rs[2:(length(Hs) - 1)]
gstar

## -------------------------------------------------------------
## plot the product of g* and px (the limit of the PMF)
## -------------------------------------------------------------
plot(x[ind], gstar[ind] * p_rs, main = expression("g"^"*"* " * p"), 
    xaxt = "n", pch = 19, col = 2, ylab = "g*", type = "b", xlab = xla)
axis(1, at = 0:100, labels = 0:100); abline(v = x[gKnots], lty = 2, 
    col = grey(0.75))

## compute Y(x) for x = r, ..., s - 1 and plot it
Ys <- rep(0, length(xs))
for (i in 2:length(xs)){
    for (ks in r:(xs[i] - 1)){
        js <- r:ks
        Ys[i] <- Ys[i] + sum(cx[js])
    }
}

## plot the two processes
plot(xs, Ys, type = "n", col = 2, xaxt = "n", lwd = 2, main = "The 
    processes H(x) and Y(x)", ylab = "H and Y", xlab = xla)
axis(1, at = 0:100, labels = 0:100); abline(v = x[gKnots], lty = 2, 
    col = grey(0.75))
lines(xs, Hs, col = 2, lwd = 1, type = "b")
lines(xs, Ys, col = 3, lwd = 2, type = "l", lty = 2)
legend("topleft", c("H(x)", "Y(x)"), col = 2:3, lty = c(1, 2), pch = 1, 
    bty = "n", lwd = c(1, 2))
}
\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}

\name{logcondiscr-package}
\alias{logcondiscr-package}
\alias{logcondiscr}
\docType{package}
\title{Estimate a Log-Concave Probability Mass Function from Discrete I.i.d. Observations}
\description{Implements the maximum likelihood estimator (MLE) for a probability mass function (PMF) under the assumption of log-concavity from i.i.d. data.}
\details{
\tabular{ll}{
Package: \tab logcondiscr \cr
Type: \tab Package \cr
Version: \tab 1.0.7 \cr
Date: \tab 2026-04-26 \cr
License: \tab GPL (>=2) \cr
LazyLoad: \tab yes \cr
}

The main functions in the package are: 

\code{\link{logConDiscrMLE}}: Compute the maximum likelihood estimator (MLE) of a log-concave PMF from i.i.d. data. The constrained log-likelihood function is maximized using an active set algorithm as initially described in Weyermann (2007). 

\code{\link{logConDiscrCI}}: Compute the maximum likelihood estimator (MLE) of a log-concave PMF from i.i.d. data and corresponding, asymptotically valid, pointwise confidence bands as
developed in Balabdaoui et al (2012). 

\code{\link{kInflatedLogConDiscr}}: Compute an estimate of a mixture of a log-concave PMF that is inflated at \eqn{k}, from i.i.d. data, using an EM algorithm. 
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Weyermann, K. (2007).
An Active Set Algorithm for Log-Concave Discrete Distributions.
\emph{MSc thesis, University of Bern} (Supervisor: Lutz Duembgen).
}
\seealso{Functions to estimate the log-concave MLE for a univariate continuous distribution are provided in the package \pkg{logcondens} and
for observations in more than one dimension in \pkg{LogConDEAD}.}
\examples{## see the help files for the abovementioned functions for examples}
\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}

\name{logConDiscrCI}
\alias{logConDiscrCI}
\title{Compute pointwise confidence bands for the log-concave MLE of a PMF}
\description{Compute pointwise confidence bands for the log-concave maximum likelihood estimate of a log-concave probability mass function based on the limiting
theory developed in Balabdaoui et al (2012).}
\usage{logConDiscrCI(dat, conf.level = 0.95, type = c("MLE", "all")[1], 
    B = 1000, output = TRUE, seed = 2011)}
\arguments{
  \item{dat}{Data to compute MLE and confidence band for.}
  \item{conf.level}{The confidence level to be used.}
  \item{type}{To compute confidence bands one theoretically needs to know the knots of the true PMF. For type \code{MLE} the knots of the MLE will be used instead
  and for type \code{all} all observations will be considered knots. The latter is conservative and gives pointwise confidence intervals that are based on standard 
  errors from a Normal approximation (the latter comes from the asymptotic theory in Balabdaoui et al, 2011).}
  \item{B}{Number of samples to be drawn to compute resampling quantiles.}
  \item{output}{If \code{TRUE}, progress of computations is output.}
  \item{seed}{Optional seed.}
}
\details{The pointwise confidence bands are based on the limiting theory in Balabdaoui et al (2011).}
\value{A list with the following components:
  \item{MLE}{The estimated MLE (simply the output list of the function \code{logConDiscrMLE} applied to \code{dat}).}
  \item{emp}{A dataframe containing two columns: the unique sorted observations and the empirical PMF.}
  \item{CIs}{The computed confidence intervals for each \eqn{x \in \{}\code{min(dat), ..., max(dat)}\eqn{\}}.}
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Weyermann, K. (2007).
An Active Set Algorithm for Log-Concave Discrete Distributions.
\emph{MSc thesis, University of Bern} (Supervisor: Lutz Duembgen).
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\note{Values outside \eqn{[0, 1]} will be clipped. As a consequence, coverage may be higher than \eqn{1 - \alpha}.}
\examples{
# -------------------------------------------------------------
# compute MLE and confidence bands for a random sample from a 
# Poisson distribution
# -------------------------------------------------------------
set.seed(2011)
x <- sort(rpois(n = 100, lambda = 2))
mle <- logConDiscrMLE(x)
psi <- mle$psi

# compute confidence bands
CIs <- logConDiscrCI(x, type = "MLE", output = TRUE, seed = 20062011)$CIs

# plot estimated PMF and log of estimate
par(mfrow = c(1, 2), las = 1)
true <- dpois(0:20, lambda = 2)
plot(mle$x, exp(psi), type = "b", col = 2, xlim = c(min(x), max(x) + 1), 
    xlab = "x", ylim = c(0, max(exp(psi), true, CIs[, 3])), ylab = "PMF", 
    main = "Estimate MLE from a Poisson", pch = 19)
legend("topright", c("truth", "MLE", "confidence bands"), col = c(4, 2, 2), 
    lty = c(1, 1, 2), pch = c(0, 19, NA), bty = "n")

# add true PMF
lines(0:20, true, type = "l", col = 4)

# add confidence bands
matlines(CIs[, 1], CIs[, 2:3], type = "l", lty = 2, col = 2)

# log-density
plot(mle$x, psi, type = "p", col = 2, xlim = c(min(x), max(x) + 1), 
    xlab = "x", ylab = "PMF", main = "Estimate MLE from a Poisson", 
    pch = 19, ylim = c(-6, log(max(exp(psi), true, CIs[, 3]))))
lines(0:20, log(true), type = "l", col = 4)

# add confidence bands
matlines(CIs[, 1], log(CIs[, 2:3]), type = "l", lty = 2, col = 2)


# -------------------------------------------------------------
# compute confidence bands when only estimate (not original data)
# are available (as a an example we simply use the estimator from
# above)
# -------------------------------------------------------------
x.est <- 0:6
est <- c(0.09, 0.30, 0.30, 0.19, 0.09, 0.02, 0.01)

# generate original data (up to given precision)
x <- rep(0:6, times = 100 * est)
}
\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}

\name{Triangular}
\alias{dTriangular}
\alias{rTriangular}
\title{Functions to compute a and simulate from a triangular probability mass function}
\description{In Balabdaoui et al (2012) the triangular density, defined as

\deqn{p_x^{a, b, c, d, e} = c(x - a) / (b - a)}

for \eqn{x \in \{a, \ldots, c\}} and 

\deqn{p_x^{a, b, c, d, e} = (e - c)(x - b) / (d - b) + c}

for \eqn{x \in \{c, \ldots, d\}}, was used to illustrate the limit process of the log-concave MLE. In order to provide the code to generate the limit process figure in Balabdaoui et al (2012, see the example in \code{\link{logConDiscrMLE}} for the code to generate that figure) the functions \code{dTriangular} and \code{rTriangular} are included in this package. Note that \code{rTriangular} uses the rejection sampling algorithm in Devroye (1987) which was specifically developed to generate random numbers from a log-concave PMF.}
\usage{
dTriangular(a, b, c, d, e)
rTriangular(n, a, b, c, d, e)
}
\arguments{
  \item{a}{Left endpoint of triangular pmf.}
  \item{b}{Mode of triangular pmf.}
  \item{c}{Height at mode.}
  \item{d}{Left endpoint.}
  \item{e}{Height at left endpoint.}
  \item{n}{Number of random numbers to generate.}
}
\value{
\code{dTriangular} returns a vector containing the value of the PMF at all values in \eqn{x \in \{a, \ldots, d\}}.
\code{rTriangular} returns a list containing the elements:
\item{rand}{Vector with generated random numbers of length \eqn{n}.}
\item{x}{Vector \eqn{(a, \ldots, d)}.}
\item{dens}{Value of the pmf at \eqn{x}.}
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Devroye, L. (1987). 
A simple generator for discrete log-concave distributions. 
\emph{Computing}, \bold{39}, 87-91.
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\note{This function is used to generate the plot of the limit process in the help file for the function \code{logConDiscrMLE}.}
\examples{

## -------------------------------------------------------------
## compute values of triangular density and simulate from it
## -------------------------------------------------------------
a <- 1
b <- 7
c <- 8
d <- 11
e <- 2
n <- 10 ^ 2

## support
x <- seq(a, d, by = 1)

## true density
dens <- dTriangular(a, b, c, d, e)
logdens <- log(dens)
rand <- rTriangular(n, a, b, c, d, e)$rand

## does the same as rTriangular()
rand2 <- sample(x = a:d, size = n, prob = dens, replace = TRUE)
}

\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}

\name{internal}
\alias{Direction}
\alias{dMLE}
\alias{GradientL}
\alias{HesseL}
\alias{J00}
\alias{J10}
\alias{J11}
\alias{J20}
\alias{LikFunk}
\alias{LocalCoarsen}
\alias{LocalConcavity}
\alias{LocalExtend}
\alias{LocalMLE}
\alias{LocalNormalize}
\alias{StepSize}
\title{Functions for estimation of a log-concave probability mass function via maximum likelihood}
\description{Internal functions for the estimation of a log-concave probability mass function.
These functions are not intended to be called by the user directly. 

\code{\link{Direction}} Compute vector that points in direction of \eqn{\max L(\psi)} via Newton step.

\code{\link{dMLE}} Compute the vector \eqn{\psi} s.t. the log-likelihood function \eqn{L}, as implemented in \code{LikFunk}, is maximized
over all PMFs (under no additional restrictions, though).

\code{\link{GradientL}} Gradient of \code{LikFunk}.

\code{\link{HesseL}} Hesse matrix of \code{LikFunk}.

\code{\link{J00}} Function introduced in Section 2.3 in Weyermann (2007), defined as

\deqn{J^{\delta_k}(\psi_k, \psi_{k+1}) := \sum_{j=0}^{\delta_k} \exp \Bigl((1-j/\delta_k)\psi_k + (j/\delta_k) \psi_{k+1} \Bigr).}

This function is used to compute the value of the log-likelihood in \code{LikFunk}.

\code{\link{J10}} Derivative of \eqn{J^{\delta_k}(\psi_k, \psi_{k+1})} w.r.t to the first argument.

\code{\link{J11}} Derivative of \eqn{J^{\delta_k}(\psi_k, \psi_{k+1})} w.r.t to both arguments.

\code{\link{J20}} Second derivative of \eqn{J^{\delta_k}(\psi_k, \psi_{k+1})} w.r.t to the first argument.

\code{\link{LikFunk}} The log-likelihood function for the discrete log-concave MLE.

\code{\link{LocalCoarsen}} Auxiliary function.

\code{\link{LocalConcavity}} Auxiliary function.

\code{\link{LocalExtend}} Auxiliary function.

\code{\link{LocalMLE}} Auxiliary function.

\code{\link{LocalNormalize}} Auxiliary function.

\code{\link{StepSize}} Auxiliary function.
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Weyermann, K. (2007).
An Active Set Algorithm for Log-Concave Discrete Distributions.
\emph{MSc thesis, University of Bern} (Supervisor: Lutz Duembgen).
}
\seealso{All these functions are used by the function \code{\link{logConDiscrMLE}}.}
\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}

\name{kInflatedLogConDiscr}
\alias{kInflatedLogConDiscr}
\title{Compute a mixture of a point mass at 0 and a log-concave probability mass function from i.i.d. data}
\description{Using an EM algorithm, compute an estimate of a mixture of a point mass at \eqn{k} and a 
log-concave probability mass function from discrete i.i.d. data.}
\usage{kInflatedLogConDiscr(x, k = 0, prec1 = 1e-10, prec2 = 1e-15, 
    itermax = 200, output = TRUE, theta0 = 0.5, p0 = NA)}
\arguments{
  \item{x}{Vector of observations.}
  \item{k}{Point at which inflation should be assumed. Must be in \eqn{x_1, x_1 + 1, \ldots, x_{n - 1}, x_n}.}
  \item{prec1}{Precision for stopping criterion.}
  \item{prec2}{Precision to remove ends of support in case weights \eqn{<}\code{prec2}.}
  \item{itermax}{Maximal number of iterations of EM algorithm.}
  \item{output}{Logical, if \code{TRUE}, progress of EM algorithm is shown.}
  \item{theta0}{Optional initialization for \eqn{\theta_0}, the point mass at \eqn{k}.}
  \item{p0}{Optional initialization for the PMF.}
}
\details{Given a vector of observations \eqn{{\bold{x}_n} = (x_1, \ldots, x_n)}{x_n = (x_1, \ldots, x_n)} from a discrete PMF with a (potential)
point mass at \eqn{k} (typically \eqn{k = 0}), \code{\link{kInflatedLogConDiscr}} computes a pmf that is a mixture between a point mass at \eqn{k} and a log-concave PMF on \eqn{x}. 
To accomplish this, an EM algorithm is used.}
\value{A list containing the following elementes:
\item{z}{The support.}
\item{f}{The estimated \eqn{k}-inflated log-concave PMF.}
\item{E(L)}{The value of the expected composite log-likelihood at the maximum.}
\item{loglik}{The value of the composite log-likelihood at the maximum.}
\item{theta}{The estimated weight at \eqn{k}.}
\item{logconc.pmf}{The log-concave part of the mixture.}
\item{logconc.z}{The support of \code{logconc.pmf}.}
}
\references{
Balabdaoui, F., Jankowski, H., Rufibach, K., and Pavlides, M. (2013).
Maximum likelihood estimation and confidence bands for a discrete log-concave distribution.
\emph{J. R. Stat. Soc. Ser. B Stat. Methodol.}, \bold{75}(4), 769--790. 

Weyermann, K. (2007).
An Active Set Algorithm for Log-Concave Discrete Distributions.
\emph{MSc thesis, University of Bern} (Supervisor: Lutz Duembgen).
}
\author{
Kaspar Rufibach (maintainer) \email{kaspar.rufibach@gmail.com} \cr \url{http://www.kasparrufibach.ch} \cr
Fadoua Balabdaoui \email{fadoua.balabdaoui@stat.math.ethz.ch} \cr \url{https://people.math.ethz.ch/~fadouab/ }  \cr
Hanna Jankowski \email{hkj@yorku.ca} \cr \url{http://www.math.yorku.ca/~hkj} \cr
Kathrin Weyermann 
}
\examples{

## -----------------------------------------------
## generate zero-inflated negative binomial sample
## -----------------------------------------------
set.seed(2011)
n <- 100
theta <- 0.05
r <- 6
p <- 0.3
x <- rnbinom(n, r, p)

## inflate at 0
x <- ifelse(runif(n) <= theta, 0, x)

## estimate log-concave MLE
fit1 <- logConDiscrMLE(x, w = NA, psi_o = NA, prec = 1e-05, output = TRUE)

## estimate zero-inflated log-concave MLE
fit2 <- kInflatedLogConDiscr(x, k = 0)

## plot the results
par(mfrow = c(1, 1), las = 1)
plot(fit1$x, exp(fit1$psi), type = "b", col = 2, xlim = range(x), xlab = "x", 
    ylim = c(0, max(exp(fit1$psi), fit2$f)), ylab = "PMF", 
    main = "Estimate MLE from a zero-inflated negative-binomial", pch = 19)
lines(fit2$z, fit2$f, type = "b", col = 4, pch = 15)

## add the true PMF we sampled from
z <- fit2$z
f.true <- theta * c(1, rep(0, length(z) - 1)) + (1 - theta) * dnbinom(z, r, p)
lines(z, f.true, col = 6, type = "b", pch = 17)

legend("topright", c("log-concave MLE", "zero-inflated log-concave MLE", 
    "true PMF"), col = c(2, 4, 6), lty = c(1, 1, 1), pch = c(19, 15, 17), 
    bty = "n")
    
\dontrun{
## -----------------------------------------------
## generate seven-inflated negative binomial sample
## -----------------------------------------------
theta <- 0.05
r <- 4
p <- 0.3
n <- 10000
x <- rnbinom(n, r, p)
x <- ifelse(runif(n) <= theta, 7, x)
x <- c(x, rep(7, 10))

## compute different estimates
zero.mle <- kInflatedLogConDiscr(x, k = 7)
mle <- logConDiscrMLE(x, output = FALSE)
f.mle <- exp(mle$psiSupp)
z<-	zero.mle$z
f1 <- theta * c(rep(0, 7 - min(x)), 1, rep(0, max(x) - 7))
f2 <- (1 - theta) * dnbinom(z, r, p)
f.true <- f1 + f2
true <- dnbinom(z, r, p)
f.fit <- zero.mle$f
xx <- sort(unique(x))
emp <- rep(0, length(z))
emp[xx - min(x) + 1] <- as.vector(table(x) / n)

## plot results
plot(z, f.true, type = "l", ylim = c(0, max(emp)), xlab = "x", 
    ylab = "PMF", main = "Illustration k-inflated estimator")
points(z, true, type = "l", lty = 2)
points(z, f.fit, type = "l", col = "red")
points(z, zero.mle$logconc.pmf, type = "l", col = "red", lty = 2)
points(min(x):max(x), f.mle, type = "l", col = "green")
points(z, emp, type = "l", col = "purple")
points(z, emp, col = "purple")
legend("topright", inset = 0.05, c("true", "true less seven", "seven-inflated", 
    "recovered", "logconc", "empirical"), lty=c(1, 2, 1, 2, 1, 1), col = c("black", 
    "black", "red", "red", "green", "purple"))

## zoom in near mode
subs <- 4:12
plot(z[subs], f.true[subs], type = "l", ylim = c(0, max(emp)), xlab = "x", 
    ylab = "PMF", main = "Illustration k-inflated estimator")
points(z[subs], true[subs], type = "l", lty = 2)
points(z[subs], f.fit[subs], type = "l", col = "red")
points(z[subs], zero.mle$logconc.pmf[subs], type = "l", col = "red", lty = 2)
points((min(x):max(x))[subs], f.mle[subs], type = "l", col = "green")
points(z[subs], emp[subs], type = "l", col = "purple")
points(z[subs], emp[subs], col = "purple")
}
}
\keyword{distribtion}
\keyword{htest}
\keyword{nonparametric}
